@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2001-2016 CS-SI. All Rights Reserved.
+* Copyright (C) 2001-2017 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -52,7 +52,6 @@
 #include <libprelude/prelude-message-id.h>
 #include <libprelude/prelude-timer.h>
 
-#include <gcrypt.h>
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
 
@@ -80,53 +79,6 @@ static gl_lock_t dh_regen_mutex = gl_lock_initializer;
 
 
 
-static int gcry_prelude_mutex_init(void **retval)
-{
-        int ret;
-        gl_lock_t *lock;
-
-        *retval = lock = malloc(sizeof(*lock));
-        if ( ! lock )
-                return ENOMEM;
-
-        ret = glthread_lock_init(lock);
-        if ( ret < 0 )
-                free(lock);
-
-        return ret;
-}
-
-
-static int gcry_prelude_mutex_destroy(void **lock)
-{
-        return glthread_lock_destroy(*lock);
-}
-
-
-
-static int gcry_prelude_mutex_lock(void **lock)
-{
-        return glthread_lock_lock((gl_lock_t *) *lock);
-}
-
-
-static int gcry_prelude_mutex_unlock(void **lock)
-{
-        return glthread_lock_unlock((gl_lock_t *) *lock);
-}
-
-
-static struct gcry_thread_cbs gcry_threads_prelude = {
-        GCRY_THREAD_OPTION_USER,
-        NULL,
-        gcry_prelude_mutex_init,
-        gcry_prelude_mutex_destroy,
-        gcry_prelude_mutex_lock,
-        gcry_prelude_mutex_unlock
-};
-
-
-
 static int dh_check_elapsed(void)
 {
         int ret;
@@ -318,7 +270,11 @@ static int handle_gnutls_error(prelude_io_t *pio, gnutls_session_t session, serv
 
         else {
                 server_generic_log_client(client, PRELUDE_LOG_WARN, "TLS error: %s.\n", gnutls_strerror(ret));
-                if ( alert_desc && (ret = gnutls_error_to_alert(ret, &level)) > 0 )
+
+                if ( gnutls_error_is_fatal(ret) )
+                        return ret;
+
+                else if ( alert_desc && (ret = gnutls_error_to_alert(ret, &level)) > 0 )
                         *alert_desc = (gnutls_alert_description_t) ret;
         }
 
@@ -467,6 +423,9 @@ int manager_auth_client(server_generic_client_t *client, prelude_io_t *pio, gnut
         gnutls_session_t session;
         int fd = prelude_io_get_fd(pio);
         prelude_connection_permission_t permission;
+        union { int fd; void *ptr; } data;
+
+        *alert = 0;
 
         /*
          * check if we already have a TLS descriptor
@@ -474,13 +433,11 @@ int manager_auth_client(server_generic_client_t *client, prelude_io_t *pio, gnut
          */
         session = prelude_io_get_fdptr(pio);
         if ( ! session ) {
-                union { int fd; void *ptr; } data;
-
                 ret = gnutls_init(&session, GNUTLS_SERVER);
                 if ( ret < 0 ) {
                         server_generic_log_client(client, PRELUDE_LOG_WARN, "error initializing TLS session: %s.\n",
                                                   gnutls_strerror(ret));
-                        return -1;
+                        return ret;
                 }
 
                 set_default_priority(session);
@@ -500,9 +457,20 @@ int manager_auth_client(server_generic_client_t *client, prelude_io_t *pio, gnut
 
         } while ( ret < 0 && (ret = handle_gnutls_error(pio, session, client, ret, alert)) == 1 );
 
-        if ( ret <= 0 )
+        if ( ret == 0 )
                 return ret;
 
+        else if ( ret < 0 ) {
+                if ( gnutls_error_is_fatal(ret) ) {
+                        data.ptr = gnutls_transport_get_ptr(session);
+                        prelude_io_set_sys_io(pio, data.fd);
+
+                        gnutls_deinit(session);
+                }
+
+                return ret;
+        }
+
         ret = verify_certificate(client, session, alert);
         if ( ret < 0 )
                 return -1;
@@ -584,7 +552,6 @@ int manager_auth_init(prelude_client_t *client, const char *tlsopts, int dh_bits
         global_dh_bits = dh_bits;
         global_dh_lifetime = dh_lifetime;
 
-        gcry_control(GCRYCTL_SET_THREAD_CBS, &gcry_threads_prelude);
         gnutls_global_init();
 
         tls_priority_init(tlsopts);
