@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 1998-2016 CS-SI. All Rights Reserved.
+* Copyright (C) 1998-2017 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -53,7 +53,7 @@
 extern manager_config_t config;
 
 prelude_client_t *manager_client;
-struct ev_loop *manager_event_loop;
+struct ev_loop *manager_event_loop, *manager_worker_loop;
 
 static char **global_argv;
 static volatile sig_atomic_t got_signal = 0;
@@ -65,14 +65,6 @@ static volatile sig_atomic_t got_signal = 0;
  */
 static RETSIGTYPE handle_signal(int sig)
 {
-        size_t i;
-
-        /*
-         * stop the sensor server.
-         */
-        for ( i = 0; i < config.nserver; i++ )
-                sensor_server_stop(config.server[i]);
-
         got_signal = sig;
 }
 
@@ -129,6 +121,14 @@ static int fill_analyzer_infos(void)
 
 static void heartbeat_cb(prelude_client_t *client, idmef_message_t *idmef)
 {
+        int ret;
+        prelude_string_t *str;
+
+        ret = idmef_heartbeat_new_messageid(idmef_message_get_heartbeat(idmef), &str);
+        if ( ret < 0 )
+                return;
+
+        prelude_ident_generate(prelude_client_get_unique_ident(client), str);
         idmef_message_process(idmef);
 }
 
@@ -142,7 +142,7 @@ static void sig_cb(struct ev_loop *loop, struct ev_signal *s, int revent)
 #endif
 
         handle_signal(s->signum);
-        ev_break(manager_event_loop, EVUNLOOP_ALL);
+        ev_break(manager_event_loop, EVBREAK_ALL);
 
         return;
 }
@@ -181,6 +181,12 @@ int main(int argc, char **argv)
                 return -1;
         }
 
+        manager_worker_loop = ev_loop_new(EVFLAG_AUTO);
+        if ( ! manager_worker_loop ) {
+                prelude_log(PRELUDE_LOG_ERR, "error creating scheduler event loop.\n");
+                return -1;
+        }
+
 
         global_argv = argv;
         prelude_option_new_root(&manager_root_optlist);
