@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2001-2016 CS-SI. All Rights Reserved.
+* Copyright (C) 2001-2017 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -633,14 +633,6 @@ server_generic_t *sensor_server_new(void)
 
 
 
-void sensor_server_stop(server_generic_t *server)
-{
-        server_generic_stop(server);
-}
-
-
-
-
 int sensor_server_add_client(server_generic_t *server, server_generic_client_t **client, prelude_connection_t *cnx)
 {
         sensor_fd_t *cdata;
