@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 1999-2016 CS-SI. All Rights Reserved.
+* Copyright (C) 1999-2017 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -94,7 +94,7 @@ extern prelude_client_t *manager_client;
 
 static struct ev_async ev_trigger;
 extern struct ev_loop *manager_event_loop;
-static volatile sig_atomic_t continue_processing = 1;
+
 
 static int send_auth_result(server_generic_client_t *client, int result)
 {
@@ -522,9 +522,7 @@ static int wait_connection(server_generic_t **server, size_t nserver)
                 ev_io_start(manager_event_loop, &server[i]->evio);
         }
 
-        while ( continue_processing )
-                ev_run(manager_event_loop, 0);
-
+        ev_run(manager_event_loop, 0);
         return 0;
 }
 
@@ -896,14 +894,6 @@ void server_generic_start(server_generic_t **server, size_t nserver)
 
 
 
-
-void server_generic_stop(server_generic_t *server)
-{
-        continue_processing = 0;
-}
-
-
-
 void server_generic_destroy(server_generic_t *server)
 {
         close(server->sock);
